﻿Type=Class
Version=4
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'################################################################################################################################################
'#																																				#
'#						Example of how to manage MidiDevices, internal and USB and integrate with the MidiSequencer								#
'#																																				#
'#															Steve Laming 2015																	#
'#																																				#
'#																																				#
'#																																				#
'#																																				#
'#																																				#
'################################################################################################################################################



'Class module
Sub Class_Globals
	
	Private pnlSDBG As Panel
	Private mAct As Activity
	Private mSeqr As MidiSequencer
	Private ReceiverMDIs As List
	Private TransmitterMDIs As List
	Private CurrentReceiverList As List
	Private CurrentTransmitterList As List
	Private lvReceivers As ListView
	Private lvTransmitters As ListView
	Private LbllvReceiversTwoLines As Label
	Private LbllvtReceiversTwoLines As Label
	Private LbllvTransmittersTwoLines As Label
	Private mShowTransmitters As Boolean
	Private pnlTransmitters As Panel
	Private USBReceiver As MidiReceiver
	Private sbUSBDelay As SeekBar
	Private LblUSBDelay As Label
	Private lvtReceivers As ListView
	Private pnlTReceivers As Panel
	Private pnlTReceiversBG As Panel
	Private USBLUT As MidiUSBLUT
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(Act As Activity,Seqr As MidiSequencer,ShowTransmitters As Boolean,DefaultTransmitter As MidiDeviceInfo)
	mAct = Act
	mSeqr = Seqr
	mAct.LoadLayout("SelectDevices")
	mShowTransmitters = ShowTransmitters
	
	If Not(mShowTransmitters) Then pnlTransmitters.Visible = False
	
	'Lists for current MidiReceivers and MidiTransmitters
	ReceiverMDIs.Initialize
	TransmitterMDIs.Initialize
	CurrentReceiverList.Initialize
	CurrentTransmitterList.Initialize
	
	'Variables to adjust label format for listviews
	LbllvReceiversTwoLines = lvReceivers.TwoLinesLayout.Label
	LbllvTransmittersTwoLines= lvTransmitters.TwoLinesLayout.Label
	LbllvtReceiversTwoLines  = lvtReceivers.TwoLinesLayout.Label
	
	'Setup the labels
	SetDisplaySingleLine
	SetDisplayTwoLines
	
	'Get the lst vaue for the USB delay from store
	LblUSBDelay.Text = Options_Static.GetUSBDelay
	sbUSBDelay.Value = LblUSBDelay.Text
	
	'Initialize the lookuptable
	USBLUT.Initialize
	
	If DefaultTransmitter <> Null Then CurrentTransmitterList.Add(DefaultTransmitter)
	
End Sub
Sub Show
	
	'Build the available device lists
	ReceiverMDIs.Clear
	TransmitterMDIs.Clear
	'Sync the current device list with the Sequencers Receiver list in case any have been removed
	CurrentReceiverList.Clear
	
	For Each MDI As MidiDeviceInfo In MidiDevice_Static.getDeviceList
		'Sort the Receivers and Transmitters
		If MDI.DevType = MidiDevice_Static.DEVICE_RECEIVER Then
			ReceiverMDIs.Add(MDI)
			If mSeqr.Receivers.IndexOf(MDI.Receiver) > -1 Then CurrentReceiverList.Add(MDI)
		Else
			If MDI.DevType = MidiDevice_Static.DEVICE_TRANSMITTER Then
				TransmitterMDIs.Add(MDI)
			End If
		End If	
	Next
	

	
	'Check if currently selected items are still valid
	For i = 0 To CurrentReceiverList.Size - 1
		MDI = CurrentReceiverList.get(i)
		If ReceiverMDIs.IndexOf(MDI) = -1 Then CurrentReceiverList.RemoveAt(i)
	Next
	
	For i = 0 To CurrentTransmitterList.Size - 1
		MDI = CurrentTransmitterList.get(i)
		If TransmitterMDIs.IndexOf(MDI) = -1 Then CurrentTransmitterList.RemoveAt(i)
	Next
	
	'Put them in the list views
	BuildReceiverDisplayList
	BuildTransmitterDisplayList
	
	'Show the layout
	pnlSDBG.Visible = True
End Sub
'Hide the layout
Sub Hide
	pnlSDBG.Visible = False
End Sub
'Maintaining a list of transmitters is the responsibility of the App using the MidiSystem (or this module)
Sub lvTransmitters_ItemClick (Position As Int, Value As Object)
	
	'Change the selection
	Dim MDI As MidiDeviceInfo = TransmitterMDIs.get(Position)
	If CurrentTransmitterList.IndexOf(MDI) = -1 Then
		AddTransmitter(MDI)
	Else
		RemoveTransmitter(MDI)
	End If
	'Display the change
	BuildTransmitterDisplayList
End Sub
Private Sub AddTransmitter(MDI As MidiDeviceInfo)
	'Add the transmitter to our current list
	CurrentTransmitterList.Add(MDI)
	'Start the transmitter, in case it does anything in the background.  This one doesn't but the USB transmitters run a thread to listen for incoming messages
	MDI.Transmitter.Start
End Sub
Private Sub RemoveTransmitter(MDI As MidiDeviceInfo)
	Dim Pos As Int = CurrentTransmitterList.IndexOf(MDI)
	'Stop any processing that the Transmitter may do in the background
	MDI.Transmitter.Stop
	If Pos > -1 Then CurrentTransmitterList.RemoveAt(Pos)
End Sub
Private Sub BuildTransmitterDisplayList
	lvTransmitters.Clear
	'Loop through each available transmitter
	For Each MDI As MidiDeviceInfo In TransmitterMDIs
		'Check if it's a USB device
		If MDI.Driver = MidiDevice_Static.DRIVER_USB Then
			'Read the Vendor Name and Device Name from the LookUpTable
			Dim VId As String = MDI.USBDeviceType.Device.VendorId
			Dim DId As String = MDI.USBDeviceType.Device.ProductId
			'Using a list with altered labels to show selected items
			If CurrentTransmitterList.IndexOf(MDI) = -1 Then
				lvTransmitters.AddSingleLine(USBLUT.VendorName(VId) & " : " & USBLUT.DeviceName(VId,DId))
			Else
				lvTransmitters.AddTwoLines(USBLUT.VendorName(VId) & " : " & USBLUT.DeviceName(VId,DId),"")
			End If
		Else
			'Display the friendly name from the MidiDeviceInfo record
			'Using a listview with altered labels to show selected items
			If CurrentTransmitterList.IndexOf(MDI) = -1 Then
				lvTransmitters.AddSingleLine(MDI.FriendlyName & " : " & MDI.Name)
			Else
				lvTransmitters.AddTwoLines(MDI.FriendlyName & " : " & MDI.Name,"")
			End If
		End If
	Next
End Sub
'Get / set Select devices panel visable
Sub getVisible As Boolean
	Return pnlSDBG.Visible
End Sub
Sub setVisible(Visible As Boolean)
	pnlSDBG.Visible = Visible
End Sub
'Is the sub menu visible
Sub ReceiverSelectVisible As Boolean
	Return pnlTReceiversBG.Visible
End Sub
'A Receivers list is maintained by the Sequencer so it know where to send midi data 
'it can be obtained by calling Sequencer.Receivers
'The getCurrentReceivers provided in this module differs in that it returns 
'the MidiDeviceInformation as opposed to just the receiver
Sub lvReceivers_ItemClick (Position As Int, Value As Object)
	Dim MDI As MidiDeviceInfo = ReceiverMDIs.get(Position) 
	If mSeqr.Receivers.IndexOf(MDI.Receiver) > -1 Then 
		RemoveReceiver(MDI)
	Else
		AddReceiver(MDI)
		'Record the USBReceiver so we can delay sending messages if needed, to sync with the internal soundcard maybe.
		'If you want to sync multiple devices differently the this logic will need changing, it currently will only delay the last selected USB receiver
		If MDI.Driver = MidiDevice_Static.DRIVER_USB Then USBReceiver = MDI.Receiver
	End If
	'Show the changes
	BuildReceiverDisplayList
End Sub
Private Sub AddReceiver(MDI As MidiDeviceInfo)
	'Tell the Sequencer about the new Receiver
	mSeqr.SetReceiver(MDI)
	'Add it to our current list
	CurrentReceiverList.Add(MDI)
	End Sub
Private Sub RemoveReceiver(MDI As MidiDeviceInfo)
	'Remove the receiver from the Sequencer
	mSeqr.RemoveReceiver(MDI)
	'And from our current list
	Dim Pos As Int = CurrentReceiverList.IndexOf(MDI)
	CurrentReceiverList.RemoveAt(Pos)
End Sub
Private Sub BuildReceiverDisplayList
	'Display changes Sub
	'Clear the listview
	lvReceivers.Clear
	'Loop through the available receivers
	For Each MDI As MidiDeviceInfo In ReceiverMDIs
		'Check if it'sa USB device
		If MDI.Driver = MidiDevice_Static.DRIVER_USB Then
			'Read the Vendor Name and Device Name from the LookUpTable
			Dim VId As String = MDI.USBDeviceType.Device.VendorId
			Dim DId As String = MDI.USBDeviceType.Device.ProductId
			'Using a listview with altered labels to show selected items
			If mSeqr.Receivers.IndexOf(MDI.Receiver) > -1 Then
				
				lvReceivers.AddTwoLines(USBLUT.VendorName(VId) & " : " & USBLUT.DeviceName(VId,DId),"")
			Else
				lvReceivers.AddSingleLine(USBLUT.VendorName(VId) & " : " & USBLUT.DeviceName(VId,DId))
			End If
		Else
			If mSeqr.Receivers.IndexOf(MDI.Receiver) > -1 Then
				lvReceivers.AddTwoLines(MDI.FriendlyName,"")
			Else
				lvReceivers.AddSingleLine(MDI.FriendlyName)
			End If
		End If
	Next
End Sub
'Get a list of the MidiDeviceInfo for currently attached MidiReceivers
Sub getCurrentReceivers As List
	Return CurrentReceiverList
End Sub
'Get a list of the MidiDeviceInfo for currently attached MidiTransmitters
Sub getCurrentTransmitters As MidiDeviceInfo
	Return CurrentTransmitterList
End Sub
'Set up the listview labels
Private Sub SetDisplayTwoLines
	'Configure Two Line ListsStatic.Display for SongList

	'Sets second line to display in the same size as single line but with different colours
	
	'Set Twoline display to highlight last song process
	
	lvReceivers.TwoLinesLayout.ItemHeight=40Dip
	lvReceivers.TwoLinesLayout.SecondLabel.Visible=False
	
	'Configure songList two Line display
	LbllvReceiversTwoLines.Color = Colors.Blue
	LbllvReceiversTwoLines.TextSize = 14
	LbllvReceiversTwoLines.TextColor=Colors.Cyan
	LbllvReceiversTwoLines.Gravity=Gravity.LEFT
	LbllvReceiversTwoLines.Gravity=Gravity.CENTER_VERTICAL
	

	'Set Twoline display to highlight last song process
	
	lvTransmitters.TwoLinesLayout.ItemHeight=40Dip
	lvTransmitters.TwoLinesLayout.SecondLabel.Visible=False
	
	'Configure songList two Line display
	LbllvTransmittersTwoLines.Color = Colors.Blue
	LbllvTransmittersTwoLines.TextSize = 14
	LbllvTransmittersTwoLines.TextColor=Colors.Cyan
	LbllvTransmittersTwoLines.Gravity=Gravity.LEFT
	LbllvTransmittersTwoLines.Gravity=Gravity.CENTER_VERTICAL
	
	'Set Twoline display to highlight last song process
	
	lvtReceivers.TwoLinesLayout.ItemHeight=40Dip
	lvtReceivers.TwoLinesLayout.SecondLabel.Visible=False
	
	'Configure songList two Line display
	LbllvtReceiversTwoLines.Color = Colors.Blue
	LbllvtReceiversTwoLines.TextSize = 14
	LbllvtReceiversTwoLines.TextColor=Colors.Cyan
	LbllvtReceiversTwoLines.Gravity=Gravity.LEFT
	LbllvtReceiversTwoLines.Gravity=Gravity.CENTER_VERTICAL
	
End Sub
Private Sub SetDisplaySingleLine

	'Standard setup for songlist two lines

	'Set Single Line display
	lvReceivers.SingleLineLayout.Label.TextSize = 14
	lvReceivers.SingleLineLayout.ItemHeight = 40Dip
	
	lvReceivers.TwoLinesLayout.ItemHeight=60DIP
	lvReceivers.TwoLinesLayout.SecondLabel.Visible=True
	
	'Configure songList two Line display
	LbllvReceiversTwoLines.TextSize = 14
	
	LbllvReceiversTwoLines.Color=Colors.Black
	LbllvReceiversTwoLines.TextColor=Colors.Green
	LbllvReceiversTwoLines.Gravity=Gravity.LEFT
	
	'Set Single Line display
	lvTransmitters.SingleLineLayout.Label.TextSize = 14
	lvTransmitters.SingleLineLayout.ItemHeight = 40Dip
	
	lvTransmitters.TwoLinesLayout.ItemHeight = 60DIP
	lvTransmitters.TwoLinesLayout.SecondLabel.Visible=True
	
	'Configure songList two Line display
	LbllvTransmittersTwoLines.TextSize = 14
	
	LbllvTransmittersTwoLines.Color=Colors.Black
	LbllvTransmittersTwoLines.TextColor=Colors.Green
	LbllvTransmittersTwoLines.Gravity=Gravity.LEFT
	
	'Set Single Line display
	lvtReceivers.SingleLineLayout.Label.TextSize = 14
	lvtReceivers.SingleLineLayout.ItemHeight = 40Dip
	
	lvtReceivers.TwoLinesLayout.ItemHeight=60DIP
	lvtReceivers.TwoLinesLayout.SecondLabel.Visible=True
	
	'Configure songList two Line display
	LbllvtReceiversTwoLines.TextSize = 14
	
	LbllvtReceiversTwoLines.Color=Colors.Black
	LbllvtReceiversTwoLines.TextColor=Colors.Green
	LbllvtReceiversTwoLines.Gravity=Gravity.LEFT
			
			
End Sub
'Gui close panel request
Sub lblBack_Click
	Hide
End Sub
'Respond to user changing the Slider to delay the USB devices
Sub sbUSBDelay_ValueChanged (Value As Int, UserChanged As Boolean)
	If USBReceiver.IsInitialized Then
		'Display the current value
		LblUSBDelay.Text = Value
		'Tell the receiver to delay
		USBReceiver.Delay = Value
		'Record the current value
		Options_Static.SetUSBDelay(Value)
	End If
End Sub
'The sub menu subroutines that control which receiver the MidiTransmitters input is sent to
Sub lvtReceivers_ItemClick (Position As Int, Value As Object)
	
	Dim MDI As MidiDeviceInfo = ReceiverMDIs.get(Position)
	Dim TMDI As MidiDeviceInfo = lvTransmitters.Tag
	If TMDI.Transmitter.Receivers.IndexOf(MDI) = -1 Then
		'Add the receiver to the selected transmitter
		
		TMDI.Transmitter.RegisterReceiverDevice(MDI)
	Else
		TMDI.Transmitter.RemoveReceiverDevice(MDI)
	End If
	'Display the change
	BuildlvtDisplay
End Sub
Sub lvTransmitters_ItemLongClick (Position As Int, Value As Object)
	lvTransmitters.Tag = TransmitterMDIs.get(Position)
	BuildlvtDisplay
	pnlTReceiversBG.Visible = True
	pnlTReceiversBG.BringToFront
End Sub
Sub BuildlvtDisplay
	lvtReceivers.Clear
	Dim TMDI As MidiDeviceInfo = lvTransmitters.Tag
	For Each MDI As MidiDeviceInfo In ReceiverMDIs
		If MDI.Driver = MidiDevice_Static.DRIVER_USB Then
			Dim VId As String = MDI.USBDeviceType.Device.VendorId
			Dim DId As String = MDI.USBDeviceType.Device.ProductId
			If TMDI.Transmitter.Receivers.indexof(MDI) > -1 Then
				lvtReceivers.AddTwoLines(USBLUT.VendorName(VId) & " : " & USBLUT.DeviceName(VId,DId),"")
			Else	
				lvtReceivers.AddSingleLine(USBLUT.VendorName(VId) & " : " & USBLUT.DeviceName(VId,DId))
			End If
		Else
			If TMDI.Transmitter.Receivers.indexof(MDI) > -1 Then
				lvtReceivers.AddTwoLines(MDI.FriendlyName,"")
			Else	
				lvtReceivers.AddSingleLine(MDI.FriendlyName)
			End If
		End If
	Next
End Sub
'Use the background panels to hide the views if the user touches outside of the main views for this display
Sub pnlSDBG_Click
	pnlSDBG.Visible = False
	Hide
End Sub
Sub pnlTReceiversBG_Click
	pnlTReceiversBG.Visible = False	
End Sub